'use strict';const utils=require('./utils'),testUtils=require('../../test/utils');describe('init utils',()=>{describe('createSocketURL',()=>{it('honors socket_protocol and uuid',()=>{const a=['ws://',window.location.hostname,':',window.location.port,'/juju/api/example.com/17070/1234-1234'].join(''),b=utils.createSocketURL({apiAddress:'http://api.example.com/',template:'/juju/api/$server/$port/$uuid',protocol:'ws',uuid:'1234-1234',server:'example.com',port:'17070'});assert.strictEqual(b,a)}),it('honors a fully qualified provided socket URL',()=>{const a=utils.createSocketURL({apiAddress:'http://api.example.com/',template:'wss://my.$server:$port/model/$uuid/api',protocol:'ws',uuid:'1234-1234',server:'example.com',port:'17070'});assert.equal(a,'wss://my.example.com:17070/model/1234-1234/api')})}),describe('unloadWindow',()=>{it('does not block when no uncommitted changes',()=>{const a={ecs:{getCurrentChangeSet:sinon.stub().returns({})}},b=utils.unloadWindow.call(a);assert.strictEqual(b,void 0)}),it('does block when has uncommitted changes',()=>{const a={ecs:{getCurrentChangeSet:sinon.stub().returns({foo:'bar'})}},b=utils.unloadWindow.call(a);assert.strictEqual(b,'You have uncommitted changes to your model. You will lose these changes if you continue.')})}),describe('humanizeTimestamp',()=>{it('must be able to display humanize time ago messages',()=>{const a=new Date().getTime();utils.humanizeTimestamp(a).should.equal('less than a minute ago'),utils.humanizeTimestamp(a+6e5).should.equal('10 minutes ago')})}),describe('_generateBundleExportFileName',()=>{it('generates the correct export file name',()=>{let a='foobar',b=new Date('October 13, 2014 11:13:00'),c=utils._generateBundleExportFileName(a,b);assert.equal(c,'foobar-2014-10-13.yaml'),a='foo-bar',b=new Date('January 13, 2014 11:13:00'),c=utils._generateBundleExportFileName(a,b),assert.equal(c,'foo-bar-2014-01-13.yaml')})}),describe('getName',()=>{it('returns the name of a charm',()=>{const a=utils.getName('cs:~uros/precise/rails-server-47');assert.strictEqual(a,'rails-server')}),it('returns the name of a charm when version is missing',()=>{const a=utils.getName('cs:~uros/precise/rails-server');assert.strictEqual(a,'rails-server')})}),describe('jujushellURL',()=>{const a=(a)=>{const b={jujushellURL:a.config||''},c={environment:{get:sinon.stub().withArgs('jujushellURL').returns(a.db||'')}},d={getItem:sinon.stub().withArgs('jujushell-url').returns(a.storage||''),setItem:sinon.stub()};return{config:b,db:c,storage:d}},b=(b,c)=>{const d=a(b),e=utils.jujushellURL(d.storage,d.db,d.config);assert.strictEqual(e,c)};it('returns the URL stored in the storage',()=>{b({storage:'wss://1.2.3.4'},'wss://1.2.3.4/ws/'),b({storage:'ws://1.2.3.4/api'},'ws://1.2.3.4/api/ws/'),b({storage:'1.2.3.4'},'wss://1.2.3.4/ws/'),b({storage:'http://1.2.3.4/'},'ws://1.2.3.4/ws/'),b({storage:'https://1.2.3.4'},'wss://1.2.3.4/ws/')}),it('returns the URL stored in the db',()=>{b({db:'shell.example.com'},'wss://shell.example.com/ws/')}),it('sets the URL in the storage if found in the db',()=>{const b=a({db:'shell.example.com'}),c=utils.jujushellURL(b.storage,b.db,b.config);assert.strictEqual(c,'wss://shell.example.com/ws/'),assert.strictEqual(b.storage.setItem.callCount,1),assert.deepEqual(b.storage.setItem.args[0],['jujushell-url','shell.example.com'])}),it('does not set the URL from db if already in the storage',()=>{const b=a({db:'db.example.com',storage:'storage.example.com'}),c=utils.jujushellURL(b.storage,b.db,b.config);assert.strictEqual(c,'wss://storage.example.com/ws/'),assert.strictEqual(b.storage.setItem.callCount,0)}),it('returns the URL stored in the config',()=>{b({config:'wss://4.3.2.1/ws'},'wss://4.3.2.1/ws')}),it('returns null if the URL cannot be found',()=>{b({},null)})}),describe('destroyService',()=>{it('responds to service removal failure by alerting the user',()=>{let a;const b='the name of the application being removed';utils._destroyServiceCallback(['service','mediawiki'],{notifications:{add:(c)=>{assert.isOk(c.title),assert.isOk(c.message),assert.notEqual(c.message.indexOf(b,-1)),assert.equal(c.level,'error'),assert.deepEqual(c.modelId,['service','mediawiki']),a=!0}}},null,{err:!0,applicationName:b}),assert.isTrue(a)}),it('removes the relations when the service is destroyed',()=>{let a=!1;const b='the name of the application being removed',c={notifications:{add:(c)=>{assert.equal(c.hasOwnProperty('title'),!0,'Does not have a title'),assert.equal(c.hasOwnProperty('message'),!0,'Does not have a message'),assert.notEqual(c.message.indexOf(b,-1)),assert.equal(c.level,'important'),a=!0}},relations:{get_relations_for_service:sinon.stub().returns([]),remove:sinon.stub()}};utils._destroyServiceCallback({},c,null,{err:!1,applicationName:b}),assert.isTrue(a),assert.equal(c.relations.remove.calledOnce,!0,'Remove relations not called')})}),describe('destroyModel',()=>{it('can destroy a model',()=>{const a=sinon.stub().callsArg(1),b={get:sinon.stub().returns('differentUUID')},c=sinon.stub(),d=sinon.stub();utils.destroyModel(a,b,c,'someuuid',d),assert.equal(a.callCount,1),assert.deepEqual(a.args[0][0],['someuuid']),assert.equal(d.callCount,1),assert.equal(c.callCount,0)}),it('disconnects when destroying the current model',()=>{const a=sinon.stub().callsArg(1),b={get:sinon.stub().returns('someuuid')},c=sinon.stub(),d=sinon.stub();utils.destroyModel(a,b,c,'someuuid',d,!1),assert.equal(a.callCount,1),assert.deepEqual(a.args[0][0],['someuuid']),assert.equal(d.callCount,1),assert.equal(c.callCount,1),assert.deepEqual(c.args[0],[null,!1,!1])})}),describe('getUnitStatusCounts',()=>{it('generate a list of status by unit counts',()=>{assert.deepEqual(utils.getUnitStatusCounts([{id:1,agent_state:'started'},{id:2,agent_state:'pending'},{id:3,agent_state:'error'},{id:4},{id:5},{id:6,agent_state:'started'},{id:7,agent_state:'error'},{id:8,agent_state:'error'},{id:9}]),{uncommitted:{priority:3,size:3},started:{priority:2,size:2},pending:{priority:1,size:1},error:{priority:0,size:3}})})}),describe('linkify',()=>{[{text:'google.com',expected:'<a href="google.com" target="_blank">google.com</a>'},{text:'www.domain.com',expected:'<a href="www.domain.com" target="_blank">www.domain.com</a>'},{text:'thisisareallylongdomainnamewithunder62parts.co',expected:'<a href="thisisareallylongdomainnamewithunder62parts.co" target="_blank">thisisareallylongdomainnamewithunder62parts.co</a>'},{text:'node-1.www4.example.com.jp',expected:'<a href="node-1.www4.example.com.jp" target="_blank">node-1.www4.example.com.jp</a>'},{text:'http://domain.com',expected:'<a href="http://domain.com" target="_blank">http://domain.com</a>'},{text:'ftp://foo.1.example.com.uk',expected:'<a href="ftp://foo.1.example.com.uk" target="_blank">ftp://foo.1.example.com.uk</a>'},{text:'example.com/?foo=bar',expected:'<a href="example.com/?foo=bar" target="_blank">example.com/?foo=bar</a>'},{text:'example.com/foo/bar?baz=true&something=%20alsotrue',expected:'<a href="example.com/foo/bar?baz=true&amp;something=%20alsotrue" target="_blank">example.com/foo/bar?baz=true&amp;something=%20alsotrue</a>'},{text:'http://example.com/index?foo=bar<script>alert(\'xss\')</script>',expected:'<a href="http://example.com/index?foo=bar&lt;script&gt;alert(\'xss\')&lt;/script&gt" target="_blank">http://example.com/index?foo=bar&lt;script&gt;alert(\'xss\')&lt;/script&gt</a>;'},{text:'http://example.com/foo"bar',expected:'<a href="http://example.com/foo&quot;bar" target="_blank">http://example.com/foo"bar</a>'},{text:'Hi there John.Bob',expected:'Hi there John.Bob'}].forEach((a)=>{it('correctly linkifies: '+a.text,()=>{const b=utils.linkify(a.text);assert.equal(b,a.expected)})})}),describe('compareSemver',()=>{it('properly compares semver values',()=>{assert.deepEqual(['1.2.3','2.0-alpha-foo-bar','4.11.6','4.2.0','1.5.19','1.5.5','1.5.5-foo','3.7.1-alpha-foo','4.1.3','2.3.1','10.5.5','5.1','11.3.0'].slice().sort(utils.compareSemver),['1.2.3','1.5.5-foo','1.5.5','1.5.19','2.0-alpha-foo-bar','2.3.1','3.7.1-alpha-foo','4.1.3','4.2.0','4.11.6','5.1','10.5.5','11.3.0'])})}),describe('switchModel',()=>{let a,b,c;beforeEach(()=>{c=utils._switchModel,a=utils._hidePopup,utils._hidePopup=sinon.stub(),b=utils._showUncommittedConfirm,utils._showUncommittedConfirm=sinon.stub(),utils._getAuth=sinon.stub().returns({rootUserName:'animal'})}),afterEach(()=>{utils._hidePopup=a,utils._showUncommittedConfirm=b,utils._switchModel=c}),it('can switch directly if there are no uncommitted changes',()=>{const a={get:sinon.stub().withArgs('modelUUID').returns('model-uuid')},b={get:sinon.stub().returns({isCommitting:sinon.stub().returns(!1),getCurrentChangeSet:sinon.stub().returns({})})},c={id:'uuid',name:'mymodel',owner:'who'};utils._switchModel=sinon.stub(),utils.switchModel.call(a,b,sinon.stub(),c),assert.deepEqual(utils._switchModel.callCount,1);const d=utils._switchModel.lastCall.args;assert.deepEqual(d,[b,c,!0])}),it('does not switch to the current model',()=>{const a={get:sinon.stub().returns({isCommitting:sinon.stub().returns(!1),getCurrentChangeSet:sinon.stub().returns({})})};utils._switchModel=sinon.stub(),utils.switchModel.call({modelUUID:'model-uuid-1'},a,sinon.stub(),{id:'model-uuid-1',name:'mymodel',owner:'who'}),assert.deepEqual(utils._switchModel.callCount,0)}),it('can show a confirmation if there are uncommitted changes',()=>{const a={get:sinon.stub().withArgs('modelUUID').returns('model-uuid')},b={get:sinon.stub().returns({isCommitting:sinon.stub().returns(!1),getCurrentChangeSet:sinon.stub().returns({change:'a change'})})};utils._switchModel=sinon.stub(),utils.switchModel.call(a,b,sinon.stub(),{id:'uuid',name:'mymodel',owner:'who'}),assert.deepEqual(utils._showUncommittedConfirm.callCount,1),assert.deepEqual(utils._switchModel.callCount,0)}),it('does not switch when committing',()=>{const a={get:sinon.stub().withArgs('modelUUID').returns('model-uuid')},b={get:sinon.stub().returns({isCommitting:sinon.stub().returns(!0),getCurrentChangeSet:sinon.stub().returns({change:'a change'})})},c=sinon.stub();utils._switchModel=sinon.stub(),utils.switchModel.call(a,b,c,{id:'uuid',name:'mymodel',owner:'who'}),assert.deepEqual(c.callCount,1),assert.deepEqual(utils._switchModel.callCount,0)}),it('allows switching to disconnected state',()=>{const a={get:sinon.stub().withArgs('modelUUID').returns('model-uuid')},b={get:sinon.stub().returns({isCommitting:sinon.stub().returns(!1),getCurrentChangeSet:sinon.stub().returns({})})};utils._switchModel=sinon.stub(),utils.switchModel.call(a,b,sinon.stub(),null),assert.deepEqual(utils._switchModel.callCount,1);const c=utils._switchModel.lastCall.args;assert.deepEqual(c,[b,null,!0])}),it('can switch models',()=>{const a={modelUUID:'uu-id',state:{changeState:sinon.stub(),current:{}}},b={set:sinon.stub()};utils._switchModel.call(a,b,{id:'my-uuid',name:'mymodel',owner:'who'}),assert.equal(utils._hidePopup.callCount,1,'_hidePopup'),assert.equal(a.state.changeState.callCount,1,'changeState'),assert.deepEqual(a.state.changeState.args[0],[{profile:null,gui:{status:null,inspector:null},root:null,hash:null,model:{path:'who/mymodel',uuid:'my-uuid'}}]),assert.equal(b.set.callCount,1,'env.set'),assert.deepEqual(b.set.args[0],['environmentName','mymodel']),assert.equal(a.modelUUID,'my-uuid')}),it('changes to disconnected mode if model is missing',()=>{const a={set:sinon.stub().withArgs('modelUUID'),state:{changeState:sinon.stub(),current:{}}},b=sinon.stub().withArgs(),c={get:sinon.stub().withArgs('ecs').returns({clear:b}),set:sinon.stub()};utils._switchModel.call(a,c,null),assert.deepEqual(a.state.changeState.args[0],[{profile:null,gui:{status:null,inspector:null},root:'new',hash:null,model:null}]),assert.equal(b.callCount,1,'clear')}),it('does not set root state to new if profile state exists',()=>{const a={set:sinon.stub().withArgs('modelUUID'),state:{current:{profile:'animal'},changeState:sinon.stub()}},b={set:sinon.stub()};utils._switchModel.call(a,b,null),assert.deepEqual(a.state.changeState.args[0],[{profile:null,gui:{status:null,inspector:null},hash:null,root:null,model:null}])}),it('does not close the status pane when switching to a model',()=>{const a={set:sinon.stub().withArgs('modelUUID'),state:{current:{gui:{status:''}},changeState:sinon.stub()}},b={set:sinon.stub()};utils._switchModel.call(a,b,{id:'my-uuid',name:'mymodel',owner:'who'}),assert.deepEqual(a.state.changeState.args[0],[{profile:null,gui:{status:'',inspector:null},hash:null,root:null,model:{path:'who/mymodel',uuid:'my-uuid'}}])}),it('closes the status pane when switching to a new model',()=>{const a={set:sinon.stub().withArgs('modelUUID'),state:{current:{gui:{status:''}},changeState:sinon.stub()}},b=sinon.stub().withArgs(),c={get:sinon.stub().withArgs('ecs').returns({clear:b}),set:sinon.stub()};utils._switchModel.call(a,c,null),assert.deepEqual(a.state.changeState.args[0],[{profile:null,gui:{status:null,inspector:null},hash:null,root:'new',model:null}]),assert.equal(b.callCount,1,'clear')})}),describe('showProfile',()=>{it('calls changeState with the supplied username',()=>{const a=sinon.stub(),b='hatch';utils.showProfile(a,b),assert.deepEqual(a.args[0],[{profile:b,root:null,store:null,search:null,user:null}])})}),describe('deploy util',()=>{let a,b,c,d,e;beforeEach(()=>{e=testUtils.makeAppContainer();const f=new window.jujugui.User({sessionStorage:(()=>{return new function(){return{store:{},setItem:function(a,b){this.store.name=b},getItem:function(){return this.store.name||null}}}})()});f.controller={user:'user',password:'password'},b=sinon.stub(),c=sinon.stub(),d=sinon.stub(),d.withArgs('ecs').returns({commit:c}),d.withArgs('connected').returns(!0),a={applicationConfig:{apiAddress:'apiAddress',socketTemplate:'socketTemplate',socket_protocol:'socket_protocol',uuid:'uuid'},modelAPI:{connect:sinon.stub(),get:d,on:sinon.stub(),set:sinon.stub()},controllerAPI:{createModel:sinon.stub()},_autoPlaceUnits:sinon.stub(),db:{notifications:{add:sinon.stub()}},set:sinon.stub(),createSocketURL:sinon.stub().returns('wss://socket-url'),get:sinon.stub().returns('wss://socket-url'),switchEnv:sinon.stub(),state:{current:{},changeState:sinon.stub()},user:f}}),afterEach(()=>{e.remove()}),it('can auto place when requested',()=>{const c=sinon.stub();utils.deploy(a,c,sinon.stub(),b,!0),assert.equal(c.callCount,1)}),it('does not auto place when requested',()=>{const c=sinon.stub();utils.deploy(a,c,sinon.stub(),b,!1),assert.equal(a._autoPlaceUnits.callCount,0)}),it('can commit to an existing model',()=>{utils.deploy(a,sinon.stub(),sinon.stub(),b),assert.equal(c.callCount,1),assert.equal(b.callCount,1),assert.equal(a.controllerAPI.createModel.callCount,0),assert.equal(a.state.changeState.callCount,1),assert.deepEqual(a.state.changeState.args[0],[{postDeploymentPanel:{show:!0}}])}),it('can create a new model',()=>{d.withArgs('connected').returns(!1),utils.deploy(a,sinon.stub(),sinon.stub(),b,!0,'new-model',{credential:'the-credential',cloud:'azure',region:'north'}),assert.equal(c.callCount,0),assert.equal(b.callCount,0),assert.equal(a.controllerAPI.createModel.callCount,1);const e=a.controllerAPI.createModel.args[0];assert.strictEqual(e[0],'new-model'),assert.strictEqual(e[1],'user@local'),assert.deepEqual(e[2],{credential:'the-credential',cloud:'azure',region:'north'}),assert.isFunction(e[3])}),it('can create, connect, and commit to the new model',()=>{const c={id:'abc123',name:'model-name',owner:'foo@external',uuid:'the-uuid'},e={model:'args'};d.withArgs('connected').returns(!1);const f=sinon.stub(),g=sinon.stub().returns('wss://socket-url');d.withArgs('ecs').returns({commit:f}),utils._hidePopup=sinon.stub(),utils.deploy(a,sinon.stub(),g,b,!1,'my-model',e),assert.equal(a.controllerAPI.createModel.callCount,1),a.controllerAPI.createModel.args[0][3](null,c),assert.equal(a.modelUUID,c.uuid),assert.equal(g.callCount,1),assert.deepEqual(g.args[0][0],{apiAddress:'apiAddress',template:'socketTemplate',protocol:'socket_protocol',uuid:c.uuid}),assert.equal(a.switchEnv.callCount,1),assert.equal(a.switchEnv.args[0][0],'wss://socket-url'),assert.strictEqual(a.switchEnv.args[0][1],null),assert.strictEqual(a.switchEnv.args[0][2],null),assert.equal(typeof a.switchEnv.args[0][3],'function'),assert.strictEqual(a.switchEnv.args[0][4],!0),assert.strictEqual(a.switchEnv.args[0][5],!1),a.switchEnv.args[0][3](e),assert.equal(f.callCount,1),assert.equal(b.callCount,1),assert.deepEqual(b.args[0],[null]),assert.equal(a.state.changeState.callCount,2),assert.deepEqual(a.state.changeState.args[0],[{postDeploymentPanel:{show:!0}}])}),it('calls changeState when deploying if state matches rules',()=>{a.state.current={root:'new',special:{dd:{id:'cs:apache'}}};d.withArgs('connected').returns(!1);const c=sinon.stub();d.withArgs('ecs').returns({commit:c}),sinon.stub(utils,'_switchModel'),utils.deploy(a,sinon.stub(),sinon.stub(),b,!1,'my-model',{model:'args'}),a.controllerAPI.createModel.args[0][3](null,{id:'abc123',name:'model-name',owner:'foo@external',uuid:'the-uuid'}),assert.equal(a.state.changeState.callCount,3),assert.deepEqual(a.state.changeState.args,[[{root:null}],[{special:{dd:null}}],[{postDeploymentPanel:{show:!0}}]])}),it('can display an error notification',()=>{d.withArgs('connected').returns(!1),utils.deploy(a,sinon.stub(),sinon.stub(),b,!1,'my-model',{model:'args'}),assert.equal(a.controllerAPI.createModel.callCount,1),a.controllerAPI.createModel.args[0][3]('it broke',{uuid:'the-uuid'}),assert.equal(a.db.notifications.add.callCount,1);const c='cannot create model: it broke';assert.deepEqual(a.db.notifications.add.args[0],[{title:c,message:c,level:'error'}]),assert.equal(b.callCount,1),assert.deepEqual(b.args[0],[c])})}),describe('generateCloudCredentialName',()=>{it('can generate a cloud credential name',()=>{assert.equal(utils.generateCloudCredentialName('azure','spinach','super-cred'),'azure_spinach_super-cred')})}),describe('getCloudProviderDetails',()=>{it('can get details for a provider',()=>{const a=utils.getCloudProviderDetails('gce');assert.equal(a.id,'google')})}),describe('validateForm',()=>{it('can validate a form with an invalid field',()=>{const a={one:{validate:sinon.stub().returns(!1)},two:{validate:sinon.stub().returns(!0)}};assert.isFalse(utils.validateForm(['one','two'],a))}),it('can validate a form with valid fields',()=>{const a={one:{validate:sinon.stub().returns(!0)},two:{validate:sinon.stub().returns(!0)}};assert.isTrue(utils.validateForm(['one','two'],a))}),it('validates all fields even if one field is invalid',()=>{const a={one:{validate:sinon.stub().returns(!1)},two:{validate:sinon.stub().returns(!0)}};utils.validateForm(['one','two'],a),assert.equal(a.one.validate.callCount,1),assert.equal(a.two.validate.callCount,1)})}),describe('parseConstraints',()=>{let a;beforeEach(()=>{a=['cpu-power','cores','cpu-cores','mem','arch','tags','root-disk']}),it('can parse constraints',()=>{assert.deepEqual(utils.parseConstraints(a,'arch=amd64 cpu-cores=2 cpu-power=10 root-disk=2048 mem=1024'),{arch:'amd64',cores:null,"cpu-cores":'2',"cpu-power":'10',mem:'1024',"root-disk":'2048',tags:null})})}),describe('generateMachineDetails',()=>{let a,b;beforeEach(()=>{a=['cpu-power','cores','cpu-cores','mem','arch','tags','root-disk'],b={filterByMachine:sinon.stub().returns([1,2,3])}}),it('can generate hardware details',()=>{assert.deepEqual(utils.generateMachineDetails(a,b,{hardware:{"cpu-cores":'2',"cpu-power":'10',mem:'1024',"root-disk":'2048'},series:'wily'}),'3 units, wily, cpu cores: 2, cpu power: 0.1GHz, mem: 1.00GB, root disk: 2.00GB')}),it('can generate details with no hardware',()=>{assert.deepEqual(utils.generateMachineDetails(a,b,{series:'wily'}),'3 units, wily, hardware details not available')}),it('can generate constraints',()=>{assert.deepEqual(utils.generateMachineDetails(a,b,{constraints:'cpu-cores=2 cpu-power=10 root-disk=2048 mem=1024',series:'wily'}),'3 units, wily, requested constraints: cpu power: 0.1GHz, cpu cores: 2, mem: 1.00GB, root disk: 2.00GB')}),it('can generate details with no constraints',()=>{assert.deepEqual(utils.generateMachineDetails(a,b,{commitStatus:'uncommitted',series:'wily'}),'3 units, wily, default constraints')}),it('can generate details with no series',()=>{assert.deepEqual(utils.generateMachineDetails(a,b,{}),'3 units, hardware details not available')})}),describe('addGhostAndEcsUnits',function(){function a(a){const b={addUnits:sinon.stub(),removeUnits:sinon.stub()},c={add_unit:sinon.stub()},d=sinon.stub(),e=utils.addGhostAndEcsUnits(b,c,{get:function(b){let c;return'id'===b?c=a:'units'===b?c={size:function(){return 2},toArray:function(){return'no-units'===a?[]:[{id:a+'/1'},{id:a+'/2'}]}}:'displayName'===b?c=0<a.indexOf('$')?'('+a+')':a:'charm'===b?c='I am a charm url':'subordinate'===b?c=!1:void 0,c}},2,d);assert.equal(b.addUnits.callCount,2,'db addUnits not called');const f=b.addUnits.args;let g=3;'no-units'===a&&(g=0),assert.deepEqual(f[0][0],{id:a+'/'+g,displayName:a+'/'+g,charmUrl:'I am a charm url',subordinate:!1},'addUnits first not called with proper data'),assert.deepEqual(f[1][0],{id:a+'/'+(g+1),displayName:a+'/'+(g+1),charmUrl:'I am a charm url',subordinate:!1},'addUnits second not called with proper data'),assert.equal(c.add_unit.callCount,2,'add unit not called');const h=c.add_unit.args;assert.equal(h[0][0],a),assert.equal(h[0][1],1),assert.strictEqual(h[0][2],null),assert.equal(typeof h[0][3],'function'),assert.deepEqual(h[0][4],{modelId:a+'/'+g}),assert.equal(h[1][0],a),assert.equal(h[1][1],1),assert.strictEqual(h[1][2],null),assert.equal(typeof h[1][3],'function'),assert.deepEqual(h[1][4],{modelId:a+'/'+(g+1)}),assert.equal(e.length,2)}it('creates machines, units; places units; updates unit lists',function(){a('myService')}),it('creates machines, units; places units; updates unit lists (no units)',function(){a('no-units')}),it('creates machines, units; places units; updates unit lists for ghosts',function(){a('myGhostService$')}),it('properly removes the ghost units on env add_unit callback',function(){const a={ghostUnit:'I am'},b={removeUnits:sinon.stub()},c=sinon.stub(),d={applicationName:'appName'};utils.removeGhostAddUnitCallback(a,b,c,d),assert.equal(b.removeUnits.calledOnce,!0),assert.equal(b.removeUnits.lastCall.args[0].service,'appName'),assert.equal(c.calledOnce,!0),assert.deepEqual(c.lastCall.args,[d,b,a])})})});